@@ -5,14 +5,40 @@
 //    _/    _/       _/             _/    _/    _/   _/
 // _/_/_/   _/_/_/  _/_/_/_/_/     _/     _/_/_/   _/_/
 // ===========================================================
-// Name        : CoreGDS
-// Author      : Xiuwen Zheng
-// Version     : 1.0.1
-// Copyright   : Xiuwen Zheng (unpublished)
-// Description :
-// ===========================================================
+//
+// CoreGDS.cpp: Export the C interface of CoreArray library
+//
+// Copyright (C) 2012	Xiuwen Zheng
+//
+// This file is part of CoreArray.
+//
+// CoreArray is free software: you can redistribute it and/or modify it
+// under the terms of the GNU Lesser General Public License Version 3 as
+// published by the Free Software Foundation.
+//
+// CoreArray is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+//
+// You should have received a copy of the GNU Lesser General Public
+// License along with CoreArray.
+// If not, see <http://www.gnu.org/licenses/>.
 
+/**
+ *	\file     CoreGDS.cpp
+ *	\author   Xiuwen Zheng
+ *	\version  1.0
+ *	\date     2007 - 2012
+ *	\brief    Export the C interface of CoreArray library
+ *	\details
+**/
+
+
+#include <CoreDEF.h>
 #include <dType.h>
+#include <dPlatform.h>
+#include <dBase.h>
 #include <dSeq.h>
 #include <dParallel.h>
 
@@ -22,18 +48,13 @@
 #include <algorithm>
 #include <map>
 
-
-#ifdef COREARRAY_WIN32
-#  ifdef COREARRAY_GNUG
-#    define DLLEXPORT __attribute__((dllexport))
-#  else
-#    define DLLEXPORT __declspec(dllexport)
-#  endif
-#else
-#  define DLLEXPORT
+#if !defined(COREARRAY_WINDOWS)
+#   undef COREARRAY_DLLEXPORT
+#   define COREARRAY_DLLEXPORT
 #endif
 
 
+
 namespace CoreArray
 {
 	// Error Object
@@ -77,7 +98,7 @@ namespace CoreArray
 			}
 			if (Stop)
 				throw ErrCoreGDS("The user stops!");
-	}
+		}
 	};
 
 
@@ -146,22 +167,19 @@ namespace CoreArray
 			}
 		}
 
-		virtual void AddDim(Int32 NewDimLen = -1) {};
-		virtual void DeleteDim() {};
-		virtual void SetDimLen(const Int32 *Lens, size_t LenCnt) {};
-		virtual void SetDLen(int DimIndex, Int32 Value) {};
-		virtual void Append(void const* Buffer, ssize_t Cnt, TSVType InSV) {};
-		virtual void AppendIter(TdIterator &iter, ssize_t Cnt) {};
-		virtual void Clear() {};
+		virtual void AddDim(Int32 NewDimLen = -1) {}
+		virtual void DeleteDim() {}
+		virtual void SetDimLen(const Int32 *Lens, size_t LenCnt) {}
+		virtual void SetDLen(int DimIndex, Int32 Value) {}
+		virtual void Append(void const* Buffer, ssize_t Cnt, TSVType InSV) {}
+		virtual void AppendIter(TdIterator &iter, ssize_t Cnt) {}
+		virtual void Clear() {}
 
-		virtual UTF16String Name() const
-		{ return UTF16String(); };
-
-		virtual int DimCnt() const
-			{ return 2; };
+		virtual UTF16String Name() const { return UTF16String(); }
 
+		virtual int DimCnt() const { return 2; }
 		virtual void GetDimLen(Int32 *Dims) const
-			{ Dims[0] = fRowCnt; Dims[1] = fColCnt; };
+			{ Dims[0] = fRowCnt; Dims[1] = fColCnt; }
 
 		virtual Int32 GetDLen(int DimIndex) const
 		{
@@ -186,7 +204,7 @@ namespace CoreArray
 			rv.Handler = this;
 			IterRow(rv) = IterCol(rv) = 0;
 			return rv;
-	}
+		}
 		virtual TdIterator atEnd()
 		{
 			TdIterator rv;
@@ -194,7 +212,8 @@ namespace CoreArray
 			IterRow(rv) = fRowCnt;
 			IterCol(rv) = 0;
 			return rv;
-	}
+		}
+
 	protected:
 		struct TItem
 		{
@@ -231,7 +250,7 @@ namespace CoreArray
 			Int32 &val = IterCol(it);
 			if ((++val) >= fColCnt)
 				{ val = 0; ++IterRow(it); }
-	}
+		}
 		virtual void _Previous(TdIterator &it)
 		{
 			Int32 &val = IterCol(it);
@@ -244,20 +263,21 @@ namespace CoreArray
 			{ return (IterRow(it)<0) || (IterRow(it)>=fRowCnt) ||
 				(IterCol(it)<0) || (IterCol(it)>=fColCnt); }
 		virtual bool _isEqual(TdIterator &it1, TdIterator &it2)
-		{ return (IterRow(it1)==IterRow(it2)) && (IterCol(it1)==IterCol(it2)); }
+			{ return (IterRow(it1)==IterRow(it2)) && (IterCol(it1)==IterCol(it2)); }
+
 	private:
-		virtual Int64 _toInt(TdIterator &it) { return 0; };
-		virtual long double _toFloat(TdIterator &it) { return 0; };
-		virtual void _IntTo(TdIterator &it, const Int64 v) {};
-		virtual void _FloatTo(TdIterator &it, const LongFloat v) {};
-		virtual void _Assign(TdIterator &it, TdIterator &its) {};
-		virtual int _Compare(TdIterator &it1, TdIterator &it2) { return 0; };
-		virtual void _LoadIter(TdIterator &it, CdFilter &Reader) {};
-		virtual void _SaveIter(TdIterator &it, CdFilter &Writer) {};
-		virtual void _InitIter(TdIterator &it, ssize_t Len) {};
-		virtual void _DoneIter(TdIterator &it, ssize_t Len) {};
-		virtual void _Swap(TdIterator &it1, TdIterator &it2) {};
-		virtual void KeepInStream(CdFilter &Reader, void * Data) {};
+		virtual Int64 _toInt(TdIterator &it) { return 0; }
+		virtual long double _toFloat(TdIterator &it) { return 0; }
+		virtual void _IntTo(TdIterator &it, const Int64 v) {}
+		virtual void _FloatTo(TdIterator &it, const LongFloat v) {}
+		virtual void _Assign(TdIterator &it, TdIterator &its) {}
+		virtual int _Compare(TdIterator &it1, TdIterator &it2) { return 0; }
+		virtual void _LoadIter(TdIterator &it, CdSerial &Reader) {}
+		virtual void _SaveIter(TdIterator &it, CdSerial &Writer) {}
+		virtual void _InitIter(TdIterator &it, ssize_t Len) {}
+		virtual void _DoneIter(TdIterator &it, ssize_t Len) {}
+		virtual void _Swap(TdIterator &it1, TdIterator &it2) {}
+		virtual void KeepInStream(CdSerial &Reader, void * Data) {}
 
 		inline Int32 &IterRow(TdIterator &I)
 			{ return *static_cast<Int32*>((void*)&I.VByte[0]); }
@@ -326,7 +346,7 @@ namespace CoreArray
 			return NULL;
 		}
 	};
-};
+}
 
 using namespace std;
 using namespace CoreArray;
@@ -363,29 +383,29 @@ extern "C"
 
 	// Functions for CoreArray System
 
-	DLLEXPORT void gds_SysInit()
+	COREARRAY_DLLEXPORT void gds_SysInit()
 	{
 		RegisterClass();
 	}
 
-	DLLEXPORT int gds_SysVersion()
+	COREARRAY_DLLEXPORT int gds_SysVersion()
 	{
 		return COREARRAY_VERSION;
 	}
 
-	DLLEXPORT const char *gds_SysVersionStr()
+	COREARRAY_DLLEXPORT const char *gds_SysVersionStr()
 	{
 		return COREARRAY_VERSION_STR;
 	}
 
-	DLLEXPORT int gds_SysClassCount()
+	COREARRAY_DLLEXPORT int gds_SysClassCount()
 	{
 		CORETRY
-	    return dObjManager().ClassMap().size();
+			return dObjManager().ClassMap().size();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_SysClassStruct(int Index, const char **OutName,
+	COREARRAY_DLLEXPORT bool gds_SysClassStruct(int Index, const char **OutName,
 		const char **OutDesp, int *OutClassType)
 	{
 		CORETRY
@@ -398,19 +418,19 @@ extern "C"
 				it++;
 			*OutName = it->first;
 			*OutDesp = it->second.Desp;
-	    *OutClassType = it->second.CType;
+			*OutClassType = it->second.CType;
 			return true;
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_SysPipeCount()
+	COREARRAY_DLLEXPORT int gds_SysPipeCount()
 	{
 		CORETRY
 			return dStreamPipeMgr.RegList().size();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_SysPipeStruct(int Index, const char **OutCoder,
+	COREARRAY_DLLEXPORT bool gds_SysPipeStruct(int Index, const char **OutCoder,
 		const char **OutDesp)
 	{
 		CORETRY
@@ -426,7 +446,7 @@ extern "C"
 
 	// Functions for File
 
-	DLLEXPORT CdGDSFile *gds_FileOpen(const char *FileName, bool ReadOnly)
+	COREARRAY_DLLEXPORT CdGDSFile *gds_FileOpen(const char *FileName, bool ReadOnly)
 	{
 		CdGDSFile *rv = new CdGDSFile;
 		CORETRY
@@ -436,7 +456,7 @@ extern "C"
 		CORECATCH2(delete rv, NULL);
 	}
 
-	DLLEXPORT CdGDSFile *gds_FileCreate(const char *FileName)
+	COREARRAY_DLLEXPORT CdGDSFile *gds_FileCreate(const char *FileName)
 	{
 		CdGDSFile *rv = new CdGDSFile;
 		CORETRY
@@ -446,7 +466,7 @@ extern "C"
 		CORECATCH2(delete rv, NULL);
 	}
 
-	DLLEXPORT bool gds_FileClose(CdGDSFile *Handle)
+	COREARRAY_DLLEXPORT bool gds_FileClose(CdGDSFile *Handle)
 	{
 		CORETRY
 		vector<CdGDSFile*>::iterator i;
@@ -459,7 +479,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_FileSync(CdGDSFile *Handle)
+	COREARRAY_DLLEXPORT bool gds_FileSync(CdGDSFile *Handle)
 	{
 		CORETRY
 			Handle->SyncFile();
@@ -467,7 +487,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_FileName(CdGDSFile *Handle, char *OutStr, int OutBufLen)
+	COREARRAY_DLLEXPORT int gds_FileName(CdGDSFile *Handle, char *OutStr, int OutBufLen)
 	{
 		CORETRY
 			string s = UTF16toUTF8(Handle->FileName());
@@ -477,12 +497,12 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_FileReadOnly(CdGDSFile *Handle)
+	COREARRAY_DLLEXPORT bool gds_FileReadOnly(CdGDSFile *Handle)
 	{
 		return Handle->ReadOnly();
 	}
 
-	DLLEXPORT CdGDSObj *gds_FilePath(CdGDSFile *Handle, const char *Path)
+	COREARRAY_DLLEXPORT CdGDSObj *gds_FilePath(CdGDSFile *Handle, const char *Path)
 	{
 		CORETRY
 			if ((Path==NULL) || (strcmp(Path, "")==0) ||
@@ -493,14 +513,14 @@ extern "C"
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT Int64 gds_FileSize(CdGDSFile *Handle)
+	COREARRAY_DLLEXPORT Int64 gds_FileSize(CdGDSFile *Handle)
 	{
 		CORETRY
 			return Handle->GetFileSize();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_FileTidyUp(CdGDSFile *Handle)
+	COREARRAY_DLLEXPORT bool gds_FileTidyUp(CdGDSFile *Handle)
 	{
 		CORETRY
 			Handle->TidyUp();
@@ -511,49 +531,49 @@ extern "C"
 
 	// Functions for File Structure
 
-	DLLEXPORT CdGDSFolder *gds_NodeRoot(CdGDSFile *Handle)
+	COREARRAY_DLLEXPORT CdGDSFolder *gds_NodeRoot(CdGDSFile *Handle)
 	{
 		CORETRY
 			return &Handle->Root();
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT CdGDSFile *gds_NodeGDSFile(CdGDSObj *Obj)
+	COREARRAY_DLLEXPORT CdGDSFile *gds_NodeGDSFile(CdGDSObj *Obj)
 	{
 		CORETRY
 		return Obj->GDSFile();
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT int gds_NodeChildCount(CdGDSFolder *Node)
+	COREARRAY_DLLEXPORT int gds_NodeChildCount(CdGDSFolder *Node)
 	{
 		CORETRY
 		return Node->Count();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT CdGDSFolder *gds_NodeChildFolder(CdGDSFolder *Node, int Index)
+	COREARRAY_DLLEXPORT CdGDSFolder *gds_NodeChildFolder(CdGDSFolder *Node, int Index)
 	{
 		CORETRY
 		return &Node->DirItem(Index);
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT CdGDSObj *gds_NodeChild(CdGDSFolder *Node, int Index)
+	COREARRAY_DLLEXPORT CdGDSObj *gds_NodeChild(CdGDSFolder *Node, int Index)
 	{
 		CORETRY
 			return Node->ObjItem(Index);
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT CdGDSObj *gds_NodePath(CdGDSFolder *Node, const char *Path)
+	COREARRAY_DLLEXPORT CdGDSObj *gds_NodePath(CdGDSFolder *Node, const char *Path)
 	{
 		CORETRY
 		return Node->Path(Path);
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT bool gds_NodeDelete(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeDelete(CdGDSObj *Node)
 	{
 		CORETRY
 			if (Node->Folder())
@@ -565,7 +585,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_NodeMoveTo(CdGDSObj *Source, CdGDSFolder *Dest)
+	COREARRAY_DLLEXPORT bool gds_NodeMoveTo(CdGDSObj *Source, CdGDSFolder *Dest)
 	{
 		CORETRY
 		Source->MoveTo(*Dest);
@@ -573,7 +593,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_NodeCopyTo(CdGDSObj *Source, CdGDSFolder *Dest)
+	COREARRAY_DLLEXPORT bool gds_NodeCopyTo(CdGDSObj *Source, CdGDSFolder *Dest)
 	{
 		CORETRY
 			UTF16String n = Source->Name();
@@ -591,14 +611,14 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT CdGDSFolder *gds_NodeAddFolder(CdGDSFolder *Folder, const char *Name)
+	COREARRAY_DLLEXPORT CdGDSFolder *gds_NodeAddFolder(CdGDSFolder *Folder, const char *Name)
 	{
 		CORETRY
 			return &(Folder->AddFolder(Name));
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT bool gds_NodeAddNull(CdGDSFolder *Folder, const char *Name)
+	COREARRAY_DLLEXPORT bool gds_NodeAddNull(CdGDSFolder *Folder, const char *Name)
 	{
 		CORETRY
 			Folder->AddObj(Name, NULL);
@@ -606,7 +626,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT CdSequenceX *gds_NodeAddArray(CdGDSFolder *Folder,
+	COREARRAY_DLLEXPORT CdSequenceX *gds_NodeAddArray(CdGDSFolder *Folder,
 		const char *Name, const char *TypeName, int DimCnt)
 	{
 		CORETRY
@@ -624,11 +644,11 @@ extern "C"
 				} else {
 					delete obj;
 					Init.LastError = string("No type name ") + TypeName;
-			return NULL;
-		}
+					return NULL;
+				}
 			} else {
-		Init.LastError = string("No type name ") + TypeName;
-		return NULL;
+				Init.LastError = string("No type name ") + TypeName;
+				return NULL;
 			}
 			if (rv)
 			{
@@ -646,44 +666,44 @@ extern "C"
 
 	// Functions for File Node
 
-	DLLEXPORT bool gds_NodeIsFolder(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeIsFolder(CdGDSObj *Node)
 	{
 		return (dynamic_cast<CdGDSFolder*>(Node) != NULL);
 	}
 
-	DLLEXPORT bool gds_NodeIsNull(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeIsNull(CdGDSObj *Node)
 	{
 		return (dynamic_cast<CdGDSNull*>(Node) != NULL);
 	}
 
-	DLLEXPORT bool gds_NodeIsContainer(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeIsContainer(CdGDSObj *Node)
 	{
 		return (dynamic_cast<CdContainer*>(Node) != NULL);
 	}
 
-	DLLEXPORT bool gds_NodeIsSequence(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeIsSequence(CdGDSObj *Node)
 	{
 		return (dynamic_cast<CdSequenceX*>(Node) != NULL);
 	}
 
-	DLLEXPORT bool gds_NodeIsVector(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeIsVector(CdGDSObj *Node)
 	{
 		return (dynamic_cast<CdVectorX*>(Node) != NULL);
 	}
 
-	DLLEXPORT bool gds_NodeIsFStr(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeIsFStr(CdGDSObj *Node)
 	{
 		return (dynamic_cast<CdFStr8*>(Node) != NULL) ||
 			(dynamic_cast<CdFStr16*>(Node) != NULL) ||
 			(dynamic_cast<CdFStr32*>(Node) != NULL);
 	}
 
-	DLLEXPORT bool gds_NodeIsStreamContainer(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeIsStreamContainer(CdGDSObj *Node)
 	{
 		return (dynamic_cast<CdGDSStreamContainer*>(Node) != NULL);
 	}
 
-	DLLEXPORT int gds_NodeName(CdGDSObj *Node, char *OutStr,
+	COREARRAY_DLLEXPORT int gds_NodeName(CdGDSObj *Node, char *OutStr,
 		int OutBufLen, bool FullName)
 	{
 		CORETRY
@@ -698,7 +718,7 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_NodeSetName(CdGDSObj *Node, char *NewName)
+	COREARRAY_DLLEXPORT bool gds_NodeSetName(CdGDSObj *Node, char *NewName)
 	{
 		CORETRY
 		Node->SetName(NewName);
@@ -706,7 +726,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_NodeClassName(CdGDSObj *Node, char *OutStr, int OutBufLen)
+	COREARRAY_DLLEXPORT int gds_NodeClassName(CdGDSObj *Node, char *OutStr, int OutBufLen)
 	{
 		CORETRY
 			string n;
@@ -717,7 +737,7 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_NodeStreamInfo(CdGDSObj *Node, Int64 *TotalIn,
+	COREARRAY_DLLEXPORT bool gds_NodeStreamInfo(CdGDSObj *Node, Int64 *TotalIn,
 		Int64 *TotalOut, const char **StreamDesp)
 	{
 		CORETRY
@@ -734,7 +754,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_SetPackedMode(CdGDSObj *Node, const char *Mode)
+	COREARRAY_DLLEXPORT bool gds_SetPackedMode(CdGDSObj *Node, const char *Mode)
 	{
 		CORETRY
 			if (dynamic_cast<CdContainer*>(Node))
@@ -743,15 +763,15 @@ extern "C"
 				if (dynamic_cast<CdGDSStreamContainer*>(Node))
 					static_cast<CdGDSStreamContainer*>(Node)->SetPackedMode(Mode);
 				else
-			throw ErrCoreGDS("No 'SetPackedMode' function!");
-		return true;
+					throw ErrCoreGDS("No 'SetPackedMode' function!");
+			return true;
 		CORECATCH(false);
 	}
 
 
 	// Functions for Specific Node
 
-	DLLEXPORT TdCombine2View *gds_NodeCombineView(CdSequenceX **List,
+	COREARRAY_DLLEXPORT TdCombine2View *gds_NodeCombineView(CdSequenceX **List,
 		bool *Trans, int ListCnt)
 	{
 		CORETRY
@@ -759,7 +779,7 @@ extern "C"
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT bool gds_NodeFree(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT bool gds_NodeFree(CdGDSObj *Node)
 	{
 		CORETRY
 			Node->Release();
@@ -770,21 +790,21 @@ extern "C"
 
 	// Functions for File Node -- TdObjAttr
 
-	DLLEXPORT int gds_AttrCount(CdGDSObj *Node)
+	COREARRAY_DLLEXPORT int gds_AttrCount(CdGDSObj *Node)
 	{
 		CORETRY
 			return Node->Attribute().Count();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT int gds_AttrNameIndex(CdGDSObj *Node, const char *Name)
+	COREARRAY_DLLEXPORT int gds_AttrNameIndex(CdGDSObj *Node, const char *Name)
 	{
 		CORETRY
-		return Node->Attribute().IndexName(Name);
-	CORECATCH(-2);
+			return Node->Attribute().IndexName(Name);
+		CORECATCH(-2);
 	}
 
-	DLLEXPORT bool gds_AttrIxAdd(CdGDSObj *Node, const char *Name,
+	COREARRAY_DLLEXPORT bool gds_AttrIxAdd(CdGDSObj *Node, const char *Name,
 		const char *Value)
 	{
 		CORETRY
@@ -794,7 +814,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_AttrIxName(CdGDSObj *Node, int Index, char *OutStr,
+	COREARRAY_DLLEXPORT int gds_AttrIxName(CdGDSObj *Node, int Index, char *OutStr,
 		int OutBufLen)
 	{
 		CORETRY
@@ -805,15 +825,15 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_AttrIxSetName(CdGDSObj *Node, int Index, const char *NewName)
+	COREARRAY_DLLEXPORT bool gds_AttrIxSetName(CdGDSObj *Node, int Index, const char *NewName)
 	{
 		CORETRY
-		Node->Attribute().SetName(Index, NewName);
-		return true;
+			Node->Attribute().SetName(Index, NewName);
+			return true;
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_AttrIxStr(CdGDSObj *Node, int Index, char *OutStr,
+	COREARRAY_DLLEXPORT int gds_AttrIxStr(CdGDSObj *Node, int Index, char *OutStr,
 		int OutBufLen)
 	{
 		CORETRY
@@ -825,7 +845,7 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_AttrIxToStr(CdGDSObj *Node, int Index, const char *Str)
+	COREARRAY_DLLEXPORT bool gds_AttrIxToStr(CdGDSObj *Node, int Index, const char *Str)
 	{
 		CORETRY
 			TdsData &D = Node->Attribute()[Index];
@@ -833,12 +853,12 @@ extern "C"
 			{
 				D.Assign(Str);
 				Node->Attribute().Changed();
-			};
+			}
 			return true;
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_AttrIxType(CdGDSObj *Node, int Index)
+	COREARRAY_DLLEXPORT int gds_AttrIxType(CdGDSObj *Node, int Index)
 	{
 		CORETRY
 			TdsData &D = Node->Attribute()[Index];
@@ -846,7 +866,7 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT const char *gds_AttrIxTypeStr(CdGDSObj *Node, int Index)
+	COREARRAY_DLLEXPORT const char *gds_AttrIxTypeStr(CdGDSObj *Node, int Index)
 	{
 		CORETRY
 			TdsData &D = Node->Attribute()[Index];
@@ -857,12 +877,12 @@ extern "C"
 
 	// Functions for CdContainer - TdIterator
 
-	DLLEXPORT CdContainer* gds_IterGetHandle(TdIterator *Iter)
+	COREARRAY_DLLEXPORT CdContainer* gds_IterGetHandle(TdIterator *Iter)
 	{
 		return Iter->Handler;
 	}
 
-	DLLEXPORT bool gds_IterAdvance(TdIterator *Iter)
+	COREARRAY_DLLEXPORT bool gds_IterAdvance(TdIterator *Iter)
 	{
 		CORETRY
 			++Iter;
@@ -870,7 +890,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_IterPrevious(TdIterator *Iter)
+	COREARRAY_DLLEXPORT bool gds_IterPrevious(TdIterator *Iter)
 	{
 		CORETRY
 			--Iter;
@@ -878,21 +898,21 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_IterInt(TdIterator *Iter)
+	COREARRAY_DLLEXPORT int gds_IterInt(TdIterator *Iter)
 	{
 		CORETRY
 			return Iter->toInt();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT double gds_IterFloat(TdIterator *Iter)
+	COREARRAY_DLLEXPORT double gds_IterFloat(TdIterator *Iter)
 	{
 		CORETRY
 			return Iter->toFloat();
 		CORECATCH(NaN);
 	}
 
-	DLLEXPORT int gds_IterStr(TdIterator *Iter, char *OutStr, int OutBufLen)
+	COREARRAY_DLLEXPORT int gds_IterStr(TdIterator *Iter, char *OutStr, int OutBufLen)
 	{
 		CORETRY
 			UTF8String s = UTF16toUTF8(Iter->toStr());
@@ -902,7 +922,7 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_IterIntTo(TdIterator *Iter, int val)
+	COREARRAY_DLLEXPORT bool gds_IterIntTo(TdIterator *Iter, int val)
 	{
 		CORETRY
 			Iter->IntTo(val);
@@ -910,7 +930,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_IterFloatTo(TdIterator *Iter, double val)
+	COREARRAY_DLLEXPORT bool gds_IterFloatTo(TdIterator *Iter, double val)
 	{
 		CORETRY
 			Iter->FloatTo(val);
@@ -918,7 +938,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_IterStrTo(TdIterator *Iter, const char *Str)
+	COREARRAY_DLLEXPORT bool gds_IterStrTo(TdIterator *Iter, const char *Str)
 	{
 		CORETRY
 			Iter->StrTo(PChartoUTF16(Str));
@@ -929,14 +949,14 @@ extern "C"
 
 	// Functions for CdSequenceX
 
-	DLLEXPORT int gds_SeqDimCnt(CdSequenceX *Obj)
+	COREARRAY_DLLEXPORT int gds_SeqDimCnt(CdSequenceX *Obj)
 	{
 		CORETRY
 			return Obj->DimCnt();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_SeqGetDim(CdSequenceX *Obj, int *OutBuf)
+	COREARRAY_DLLEXPORT bool gds_SeqGetDim(CdSequenceX *Obj, int *OutBuf)
 	{
 		CORETRY
 			CdSequenceX::TSeqDimBuf buf;
@@ -948,28 +968,28 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT Int64 gds_SeqGetCount(CdSequenceX *Obj)
+	COREARRAY_DLLEXPORT Int64 gds_SeqGetCount(CdSequenceX *Obj)
 	{
 		CORETRY
 			return Obj->Count();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT int gds_SeqSVType(CdSequenceX *Obj)
+	COREARRAY_DLLEXPORT int gds_SeqSVType(CdSequenceX *Obj)
 	{
 		CORETRY
 			return Obj->SVType();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT int gds_SeqBitOf(CdSequenceX *Obj)
+	COREARRAY_DLLEXPORT int gds_SeqBitOf(CdSequenceX *Obj)
 	{
 		CORETRY
 			return Obj->BitOf();
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT bool gds_SeqIndexIter(CdSequenceX *Obj, int *Index, TdIterator *Out)
+	COREARRAY_DLLEXPORT bool gds_SeqIndexIter(CdSequenceX *Obj, int *Index, TdIterator *Out)
 	{
 		CORETRY
 			CdSequenceX::TSeqDimBuf buf;
@@ -981,7 +1001,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_SeqLoad(CdSequenceX *Obj, const char *FileName,
+	COREARRAY_DLLEXPORT bool gds_SeqLoad(CdSequenceX *Obj, const char *FileName,
 		void *Data = NULL,
 		OnGDSStreamEvent evBegin = NULL,
 		OnGDSStreamEvent evRunning = NULL,
@@ -1008,7 +1028,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT bool gds_SeqSave(CdSequenceX *Obj, const char *FileName,
+	COREARRAY_DLLEXPORT bool gds_SeqSave(CdSequenceX *Obj, const char *FileName,
 		void *Data = NULL,
 		OnGDSStreamEvent evBegin = NULL,
 		OnGDSStreamEvent evRunning = NULL,
@@ -1034,7 +1054,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT int gds_SeqFStrMaxLen(CdSequenceX *Obj)
+	COREARRAY_DLLEXPORT int gds_SeqFStrMaxLen(CdSequenceX *Obj)
 	{
 		CORETRY
 			if (dynamic_cast<CdFStr8*>(Obj))
@@ -1052,7 +1072,7 @@ extern "C"
 
 	// CdSequenceX -- read
 
-	DLLEXPORT bool gds_rData(CdSequenceX *obj, Int32 const* Start,
+	COREARRAY_DLLEXPORT bool gds_rData(CdSequenceX *obj, Int32 const* Start,
 		Int32 const* Length, void *OutBuf, TSVType OutSV)
 	{
 		CORETRY
@@ -1060,7 +1080,7 @@ extern "C"
 			return true;
 		CORECATCH(false);
 	}
-	DLLEXPORT bool gds_rDataEx(CdSequenceX *obj, Int32 const* Start,
+	COREARRAY_DLLEXPORT bool gds_rDataEx(CdSequenceX *obj, Int32 const* Start,
 		Int32 const* Length, CBOOL *Selection[], void *OutBuf, TSVType OutSV)
 	{
 		CORETRY
@@ -1071,7 +1091,7 @@ extern "C"
 
 	// CdSequenceX -- write
 
-	DLLEXPORT bool gds_wData(CdSequenceX *obj, Int32 const* Start,
+	COREARRAY_DLLEXPORT bool gds_wData(CdSequenceX *obj, Int32 const* Start,
 		Int32 const* Length, const void *InBuf, TSVType InSV)
 	{
 		CORETRY
@@ -1082,7 +1102,7 @@ extern "C"
 
 	// CdSequenceX -- append
 
-	DLLEXPORT bool gds_AppendData(CdSequenceX *obj, int Cnt, const void *InBuf, TSVType InSV)
+	COREARRAY_DLLEXPORT bool gds_AppendData(CdSequenceX *obj, int Cnt, const void *InBuf, TSVType InSV)
 	{
 		CORETRY
 			obj->Append(InBuf, Cnt, InSV);
@@ -1094,7 +1114,7 @@ extern "C"
 
 	// Functions for CdGDSStreamContainer
 
-	DLLEXPORT CdGDSStreamContainer *gds_NewContainer(CdGDSFolder *Folder,
+	COREARRAY_DLLEXPORT CdGDSStreamContainer *gds_NewContainer(CdGDSFolder *Folder,
 		const char *Name, const char *InputFile, const char *PackMode)
 	{
 		CORETRY
@@ -1109,7 +1129,7 @@ extern "C"
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT bool gds_SaveContainer(CdGDSStreamContainer *Container,
+	COREARRAY_DLLEXPORT bool gds_SaveContainer(CdGDSStreamContainer *Container,
 		const char *OutputFile)
     {
 		CORETRY
@@ -1120,7 +1140,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT Int64 gds_ContainerGetSize(CdGDSStreamContainer *Container)
+	COREARRAY_DLLEXPORT Int64 gds_ContainerGetSize(CdGDSStreamContainer *Container)
     {
 		CORETRY
 			return Container->GetSize();
@@ -1130,7 +1150,7 @@ extern "C"
 
 	// Diagnosis
 
-	DLLEXPORT int gds_DiagnBlockCount(CdGDSFile *Handle)
+	COREARRAY_DLLEXPORT int gds_DiagnBlockCount(CdGDSFile *Handle)
 	{
 		CORETRY
 			CdBlockCollection *H = (CdBlockCollection*)Handle;
@@ -1138,7 +1158,7 @@ extern "C"
 		CORECATCH(-1);
 	}
 
-	DLLEXPORT const CdBlockStream::TBlockInfo *gds_DiagnBlockInfo(
+	COREARRAY_DLLEXPORT const CdBlockStream::TBlockInfo *gds_DiagnBlockInfo(
 		CdGDSFile *Handle, int Index)
 	{
 		CORETRY
@@ -1147,7 +1167,7 @@ extern "C"
 		CORECATCH(NULL);
 	}
 
-	DLLEXPORT bool gds_DiagnBlockSize(CdGDSFile *Handle, int Index,
+	COREARRAY_DLLEXPORT bool gds_DiagnBlockSize(CdGDSFile *Handle, int Index,
 		Int64 &size, Int64 &capacity)
 	{
 		CORETRY
@@ -1158,7 +1178,7 @@ extern "C"
 		CORECATCH(false);
 	}
 
-	DLLEXPORT const CdBlockStream::TBlockInfo *gds_DiagnUnusedBlock(
+	COREARRAY_DLLEXPORT const CdBlockStream::TBlockInfo *gds_DiagnUnusedBlock(
 		CdGDSFile *Handle)
 	{
 		CORETRY
@@ -1169,12 +1189,12 @@ extern "C"
 
 
 	// Functions for Error
-	DLLEXPORT const char *gds_Error()
+	COREARRAY_DLLEXPORT const char *gds_Error()
 	{
 		return Init.LastError.c_str();
 	}
 
-	DLLEXPORT string & gds_LastError()
+	COREARRAY_DLLEXPORT string & gds_LastError()
 	{
 		return Init.LastError;
 	}
@@ -1186,66 +1206,66 @@ extern "C"
 	//
 
 	/// Return NaN
-	DLLEXPORT float conf_F32_NaN()
+	COREARRAY_DLLEXPORT float conf_F32_NaN()
 	{
 		return NaN;
 	}
 	/// Return NaN
-	DLLEXPORT double conf_F64_NaN()
+	COREARRAY_DLLEXPORT double conf_F64_NaN()
 	{
 		return NaN;
 	}
 
 	/// Return infinity
-	DLLEXPORT float conf_F32_Inf()
+	COREARRAY_DLLEXPORT float conf_F32_Inf()
 	{
 		return Infinity;
 	}
 	/// Return infinity
-	DLLEXPORT double conf_F64_Inf()
+	COREARRAY_DLLEXPORT double conf_F64_Inf()
 	{
 		return Infinity;
 	}
 
 	/// Return negative infinity
-	DLLEXPORT float conf_F32_NegInf()
+	COREARRAY_DLLEXPORT float conf_F32_NegInf()
 	{
 		return NegInfinity;
 	}
 	/// Return negative infinity
-	DLLEXPORT double conf_F64_NegInf()
+	COREARRAY_DLLEXPORT double conf_F64_NegInf()
 	{
 		return NegInfinity;
 	}
 
 	/// Return whether it is a finite number
-	DLLEXPORT bool conf_IsFinite32(float val)
+	COREARRAY_DLLEXPORT bool conf_IsFinite32(float val)
 	{
 		return IsFinite(val);
 	}
 	/// Return whether it is a finite number
-	DLLEXPORT bool conf_IsFinite64(double val)
+	COREARRAY_DLLEXPORT bool conf_IsFinite64(double val)
 	{
 		return IsFinite(val);
 	}
 
 	/// Return the number of available CPU cores in the system
 	/** return -1, if unable to determine. **/
-	DLLEXPORT int conf_GetNumberOfCPU()
+	COREARRAY_DLLEXPORT int conf_GetNumberOfCPU()
 	{
 		return Mach::GetNumberOfCPU();
 	}
 
 	/// Return the size in byte of level-1 cache memory
     /** return -1, if unable to determine. **/
-    DLLEXPORT int conf_GetL1CacheMemory()
+    COREARRAY_DLLEXPORT int conf_GetL1CacheMemory()
 	{
 		return Mach::GetL1CacheMemory();
 	}
 
 	/// Return the size in byte of level-2 cache memory
     /** return -1, if unable to determine. **/
-    DLLEXPORT int conf_GetL2CacheMemory()
+    COREARRAY_DLLEXPORT int conf_GetL2CacheMemory()
 	{
 		return Mach::GetL2CacheMemory();
 	}
@@ -1259,14 +1279,14 @@ extern "C"
 	// thread mutex
 
 	/// create a mutex object
-	DLLEXPORT CdThreadMutex* plc_InitMutex()
+	COREARRAY_DLLEXPORT CdThreadMutex* plc_InitMutex()
 	{
 		CORETRY
 			return new CdThreadMutex;
 		CORECATCH(NULL);
 	}
 	/// destroy the mutex object
-	DLLEXPORT bool plc_DoneMutex(CdThreadMutex* obj)
+	COREARRAY_DLLEXPORT bool plc_DoneMutex(CdThreadMutex* obj)
 	{
 		CORETRY
 			if (obj) delete obj;
@@ -1274,7 +1294,7 @@ extern "C"
 		CORECATCH(false);
 	}
 	/// lock the mutex object
-	DLLEXPORT bool plc_LockMutex(CdThreadMutex* obj)
+	COREARRAY_DLLEXPORT bool plc_LockMutex(CdThreadMutex* obj)
 	{
 		CORETRY
 			if (obj) obj->Lock();
@@ -1282,7 +1302,7 @@ extern "C"
 		CORECATCH(false);
 	}
 	/// unlock the mutex object
-	DLLEXPORT bool plc_UnlockMutex(CdThreadMutex* obj)
+	COREARRAY_DLLEXPORT bool plc_UnlockMutex(CdThreadMutex* obj)
 	{
 		CORETRY
 			if (obj) obj->Unlock();
@@ -1293,14 +1313,14 @@ extern "C"
 	// thread suspending object
 
 	/// initialize a thread suspending object
-	DLLEXPORT CdThreadsSuspending* plc_InitSuspend()
+	COREARRAY_DLLEXPORT CdThreadsSuspending* plc_InitSuspend()
 	{
 		CORETRY
 			return new CdThreadsSuspending;
 		CORECATCH(NULL);
 	}
 	/// destroy the thread suspending object
-	DLLEXPORT bool plc_DoneSuspend(CdThreadsSuspending* obj)
+	COREARRAY_DLLEXPORT bool plc_DoneSuspend(CdThreadsSuspending* obj)
 	{
 		CORETRY
 			if (obj) delete obj;
@@ -1308,7 +1328,7 @@ extern "C"
 		CORECATCH(false);
 	}
 	/// suspend the thread suspending object
-	DLLEXPORT bool plc_Suspend(CdThreadsSuspending* obj)
+	COREARRAY_DLLEXPORT bool plc_Suspend(CdThreadsSuspending* obj)
 	{
 		CORETRY
 			if (obj) obj->Suspend();
@@ -1316,7 +1336,7 @@ extern "C"
 		CORECATCH(false);
 	}
 	/// wakeup the thread suspending object
-	DLLEXPORT bool plc_WakeUp(CdThreadsSuspending* obj)
+	COREARRAY_DLLEXPORT bool plc_WakeUp(CdThreadsSuspending* obj)
 	{
 		CORETRY
 			if (obj) obj->WakeUp();
@@ -1327,14 +1347,14 @@ extern "C"
 
 
 
-	static CparallelBase _parallelBase;
+	static CParallelBase _ParallelBase;
 
-	DLLEXPORT bool plc_DoBaseThread(void (*Proc)(CdThread *, int, void*),
+	COREARRAY_DLLEXPORT bool plc_DoBaseThread(void (*Proc)(CdThread *, int, void*),
 		void *param, int nThread)
 	{
 		CORETRY
-			_parallelBase.SetnThread(nThread);
-			_parallelBase.DoThreads(Proc, param);
+			_ParallelBase.SetnThread(nThread);
+			_ParallelBase.DoThreads(Proc, param);
 			return true;
 		CORECATCH(false);
 	}
